# -*- coding: utf-8 -*-

from models import Message, Room
from django.contrib import admin

class RoomAdmin(admin.ModelAdmin):
    list_display = ('name', 'created', 'last_activity_formatted', 'description', 'description_modified')
    readonly_fields = ('created', 'description_modified', 'content_type', 'object_id')
    # As we've set some fields to be read-only, they will automatically appear at the end
    # of the list. Manually order the fields to be as they are defined in the model.
    fieldsets = (
        (None, {
            'fields': ('name', 'created', 'description', 'description_modified',
                       'content_type', 'object_id')
        }),
    )

    # When changing a description, we need to know the request.user as an attribute
    # of the room instance. This snippet below adds it.
    def save_model(self, request, obj, form, change):
        obj.user = request.user
        obj.save()

admin.site.register(Room, RoomAdmin)

class MessageAdmin(admin.ModelAdmin):
    list_display = ('room', 'created', 'unix_timestamp', 'user', 'text', 'event')
    list_filter = ['room', 'user']

admin.site.register(Message, MessageAdmin)




# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.safestring import mark_safe
from django.conf import settings

import datetime
import time

class Room(models.Model):
    """Conversations can take place in one of many rooms.

    >>> l = Room(name='Test room')
    >>> l.save()
    >>> l
    <Room: Test room>

    Note that updating 'description' auto-updates 'description_modified' when saving:

    >>> l.description_modified

    >>> l.description = 'A description'

    Note that we need to always set the 'user' attribute as a system message is generated for each change.
    >>> l.user = User.objects.get(id=1)
    >>> l.save()

    # description_modified is a unix timestamp.
    >>> m = l.description_modified
    >>> m > 0
    True

    """
    name = models.CharField(max_length=255, null=True, blank=True, help_text='Name of the room.')
    created = models.DateTimeField(editable=False)
    description = models.CharField(max_length=100, null=True, blank=True, help_text='The description of this room.')
    description_modified = models.IntegerField(null=True, editable=False, help_text='Unix timestamp when the description was created or last modified.')
    last_activity = models.IntegerField(editable=False,
                                        help_text='Last activity in the room. Stored as a Unix timestamp.')
    # Define generic relation to target object. This is optional.
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = generic.GenericForeignKey()

    def __unicode__(self):
        return u'%s' % (self.name)

    class Meta:
        ordering = ['created']

    def __init__(self, *args, **kw):
        super(Room, self).__init__(*args, **kw)
        self._init_description = self.description

    def save(self, **kw):
        # If description modified, update the timestamp field.
        if self._init_description != self.description:
            self.description_modified = time.time()
        # if last_activity is null (i.e. we are creating the room) set it to now.
        if not self.last_activity:
            self.last_activity = time.time()
        if not self.created:
            self.created = datetime.datetime.now()
        super(Room, self).save(**kw)

    @property
    def last_activity_formatted(self):
        """Return Unix timestamp, then express it as a time."""
        return display_timestamp(self.last_activity)

    @property
    def last_activity_datetime(self):
        """Convert last_activity into a datetime object (used to feed into timesince
        filter tag, ideally I should send a patch to Django to accept Unix times)"""
        return datetime.datetime.fromtimestamp(self.last_activity)

# The list of events can be customised for each project.
try:
    EVENT_CHOICES = settings.JQCHAT_EVENT_CHOICES
except AttributeError:
    # Use default event list.
    EVENT_CHOICES = (
                  (1, "has changed the room's description."),
                  (2, "has joined the room."),
                  (3, "has left the room."),
                 )
class messageManager(models.Manager):

    def create_message(self, user, room, msg):
        """Create a message for the given user."""
        m = Message.objects.create(user=user,
                                   room=room,
                                   text='<strong>%s</strong> %s<br />' % (user, msg))
        return m

    def create_event(self, user, room, event_id):
        """Create an event for the given user."""
        m = Message(user=user,
                    room=room,
                    event=event_id)
        m.text = '<strong>%s</strong> <em>%s</em><br />' % (user, m.get_event_display())
        m.save()
        return m

class Message(models.Model):
    """Messages displayed in the chat client.

    Note that we have 2 categories of messages:
    - a text typed in by the user.
    - an event carried out in the room ("user X has left the room.").

    New messages should be created through the supplied manager methods, as all
    messages get preformatted (added markup) for display in the chat window.
    For example:

    Messages:
    >>> user = User.objects.create_user('john', 'lennon@thebeatles.com', 'johnpassword')
    >>> room = Room.objects.create(name='Test room')
    >>> m = Message.objects.create_message(user, room, 'hello there')
    >>> m.text
    '<strong>john</strong> hello there<br />'

    Events:
    >>> m1 = Message.objects.create_event(user, room, 1)
    >>> u'<strong>john</strong> <em>has changed' in m1.text
    True

    Note that there are 2 timestamp fields:
    - a unix timestamp.
    - a datetime timestamp.
    The reason: the unix timestamp is higher performance when sending data to the browser (easier
    and faster to handle numbers instead of datetimes. The 'created' is used for displaying the date
    of messages; I could calculate it from the unix timestamp, but I'm guessing that I will get
    higher performance by storing it in the database.

    """

    user = models.ForeignKey(User, related_name='jchat_messages')
    room = models.ForeignKey(Room, help_text='This message was posted in a given chat room.')
    event = models.IntegerField(null=True, blank=True, choices=EVENT_CHOICES, help_text='An action performed in the room, either by a user or by the system (e.g. XYZ leaves room.')
    text = models.TextField(null=True, blank=True, help_text='A message, either typed in by a user or generated by the system.')
    unix_timestamp = models.FloatField(editable=False, help_text='Unix timestamp when this message was inserted into the database.')
    created = models.DateTimeField(editable=False)

    def __unicode__(self):
        return u'%s, %s' % (self.user, self.unix_timestamp)

    def save(self, **kw):
        if not self.unix_timestamp:
            self.unix_timestamp = time.time()
            self.created = datetime.datetime.fromtimestamp(self.unix_timestamp)
        super(Message, self).save(**kw)
        self.room.last_activity = int(time.time())
        self.room.save()

    class Meta:
        ordering = ['unix_timestamp']

    objects = messageManager()

def display_timestamp(t):
        """Takes a Unix timestamp as a an arg, returns a text string with
        '<unix timestamp> (<equivalent time>)'."""
        return '%s (%s)' % (t, time.strftime('%d/%m/%Y %H:%M', time.gmtime(t)))





# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *

import views

urlpatterns = patterns('',
    # Example chat room.
    url(r"room/(?P<id>\d+)/$", views.window, name="jqchat_test_window"),
    url(r"room/(?P<id>\d+)/ajax/$", views.BasicAjaxHandler, name="jqchat_ajax"),
    # Second example room - adds room descriptions.
    url(r"room_with_description/(?P<id>\d+)/$", views.WindowWithDescription, name="jqchat_test_window_with_description"),
    url(r"room_with_description/(?P<id>\d+)/ajax/$", views.WindowWithDescriptionAjaxHandler, name="jqchat_test_window_with_description_ajax"),
)






from setuptools import setup, find_packages

import jqchat

setup(
    name='django-jqchat',
    author='Richard Barran',
    author_email='richardb...@gmail.com',
    version=".".join(map(str, jqchat.VERSION)),
    url="https://github.com/tomscytale/django-jqchat",
    install_requires=[
        'Django>=1.0',
        'django-timezones',
        'pytz',
        'south',
        ],
    description='An extensible instant messenger thingy for Django',
    packages=find_packages(),
    include_package_data=True,
    zip_safe = False,
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development"
        ],
    )



from django.http import HttpResponseBadRequest
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.utils.html import escape

from models import Room, Message

import time

# The format of the date displayed in the chat window can be customised.
try:
    DATE_FORMAT = settings.JQCHAT_DATE_FORMAT
except AttributeError:
    # Use default format.
    DATE_FORMAT = "D-H:i:s"

# How many messages to retrieve at most.
JQCHAT_DISPLAY_COUNT = getattr(settings, 'JQCHAT_DISPLAY_COUNT', 100)

# Time in hours over which the messages will not be displayed
JQCHAT_DISPLAY_TIME = getattr(settings, 'JQCHAT_DISPLAY_TIME', 0)

#------------------------------------------------------------------------------
@login_required
def window(request, id):
    """A basic chat client window."""

    ThisRoom = get_object_or_404(Room, id=id)

    return render_to_response('jqchat/chat_test.html', {'room': ThisRoom},
                              context_instance=RequestContext(request))

#------------------------------------------------------------------------------
@login_required
def WindowWithDescription(request, id):
    """A variant of the basic chat window, includes an updatable description to demonstrate
    how to extend the chat system."""

    ThisRoom = get_object_or_404(Room, id=id)

    return render_to_response('jqchat/chat_test_with_desc.html', {'room': ThisRoom},
                              context_instance=RequestContext(request))

#------------------------------------------------------------------------------
class Ajax(object):
    """Connections from the jQuery chat client come here.

    We receive here 2 types of calls:
    - requests for any new messages.
    - posting new user messages.

    Any new messages are always returned (even if/when posting new data).

    Requests for new messages should be sent as a GET with as arguments:
    - current UNIX system time on the server. This is used so that the server which messages have
    already been received by the client.
    On the first call, this should be set to 0, thereafter the server will supply a new system time
    on each call.
    - the room ID number.

    Requests that include new data for the server (e.g. new messages) should be sent as a POST and
    contain the following extra args:
    - an action code, a short string describing the type of data sent.
    - message, a string containing the message sent by the user.

    The returned data contains a status flag:
     1: got new data.
     2: no new data, nothing to update.

    This code is written as a class, the idea being that implementations of a chat window will
    have extra features, so these will be coded as derived classes.
    Included below is a basic example for updating the room's description field.

    """

    # Note that login_required decorators cannot be attached here if the __call__ is to be overridden.
    # Instead they have to be attached to child classes.
    def __call__(self, request, id):

        if not request.user.is_authenticated():
            return HttpResponseBadRequest('You need to be logged in to access the chat system.')

        StatusCode = 0 # Default status code is 0 i.e. no new data.
        self.request = request
        try:
            self.request_time = float(self.request.REQUEST['time'])
        except (ValueError, TypeError, KeyError):
            return HttpResponseBadRequest("What's the time?")
        self.ThisRoom = Room.objects.get(id=id)
        NewDescription = None

        if self.request.method == "POST":
            # User has sent new data.
            action = self.request.POST['action']

            if action == 'postmsg':
                msg_text = self.request.POST['message']

                if len(msg_text.strip()) > 0: # Ignore empty strings.
                    Message.objects.create_message(self.request.user, self.ThisRoom, escape(msg_text))
        else:
            # If a GET, make sure that no action was specified.
            if self.request.GET.get('action', None):
                return HttpResponseBadRequest('Need to POST if you want to send data.')

        # If using Pinax we can get the user's timezone.
        try:
            user_tz = self.request.user.account_set.all()[0].timezone
        except (AttributeError, IndexError):
            user_tz = settings.TIME_ZONE

        # Extra JSON string to be spliced into the response.
        CustomPayload = self.ExtraHandling()
        if CustomPayload:
            StatusCode = 1

        # Get new messages - do this last in case the ExtraHandling has itself generated
        # new messages.
        NewMessages = self.ThisRoom.message_set.filter(unix_timestamp__gt=self.request_time)

        if JQCHAT_DISPLAY_TIME:
            min_unix_timestamp = time.time() - JQCHAT_DISPLAY_TIME*3600
            NewMessages = NewMessages.filter(unix_timestamp__gt=min_unix_timestamp)

        if NewMessages:
            StatusCode = 1

        # Only keep the last X messages.
        l = len(NewMessages)
        if l > JQCHAT_DISPLAY_COUNT:
            NewMessages = NewMessages[l-JQCHAT_DISPLAY_COUNT:]

        response = render_to_response('jqchat/chat_payload.json',
                                  {'current_unix_timestamp': time.time(),
                                   'NewMessages': NewMessages,
                                   'StatusCode': StatusCode,
                                   'NewDescription': NewDescription,
                                   'user_tz': user_tz,
                                   'CustomPayload': CustomPayload,
                                   'TimeDisplayFormat': DATE_FORMAT
                                   },
                                  context_instance=RequestContext(self.request))
        response['Content-Type'] = 'text/plain; charset=utf-8'
        response['Cache-Control'] = 'no-cache'
        return response

    def ExtraHandling(self):
        """We might want to receive/send extra data in the Ajax calls.
        This function is there to be overriden in child classes.

        Basic usage is to generate the JSON that then gets spliced into the main JSON
        response.

        """
        return None


BasicAjaxHandler = Ajax()


#------------------------------------------------------------------------------
class DescriptionAjax(Ajax):
    """Example of how to handle calls with extra data (in this case, a room
    description field).
    """

    def ExtraHandling(self):
        # Check if new description sent.
        if self.request.method == "POST":
            action = self.request.POST['action']
            if action == 'change_description':
                # Note that we escape descriptions as a protection against XSS.
                self.ThisRoom.description = escape(self.request.POST['description'])
                self.ThisRoom.save()
                Message.objects.create_event(self.request.user, self.ThisRoom, 1)
        # Is there a description more recent than the timestamp sent by the client?
        # If yes, return an extra field to be tagged on to the JSON returned to the client.
        if self.ThisRoom.description and self.ThisRoom.description_modified > self.request_time:
            return ',\n        "description": "%s"' % self.ThisRoom.description

        return None

WindowWithDescriptionAjaxHandler = DescriptionAjax()





"""
Unit tests for jqchat application.

"""

from django.test import TestCase
from django.test.client import Client
from django.contrib.auth.models import User
#from django.conf import settings
from django.core.urlresolvers import reverse

import simplejson
from models import Room, Message

# During testing we want to use default settings.
import views
views.DATE_FORMAT = "H:i:s"

class ChatPageTest(TestCase):
    """Test the chat test page."""

    # This includes a sample room.
    fixtures = ['test_jqchat.json']

    def setUp(self):
        self.client = Client()
        self.client.login(username='mickey', password='test')

    def test_chat_page(self):
        """We have a basic test page for the chat client."""

        # Get the room defined in the fixture.
        response = self.client.get(reverse('jqchat_test_window', kwargs={'id': 1}))
        self.assert_(response.status_code == 200, response.status_code)

        # Basic checks on the content of this page.
        self.assert_('<div id="chatwindow">' in response.content)
        self.assert_(response.context[0]['room'].name == 'The first room', response.context[0]['room'])

    def test_login(self):
        """Must be logged in to access the chat page."""

        self.client.logout()
        response = self.client.get(reverse('jqchat_test_window', kwargs={'id': 1}))
        # Should redirect to login page.
        self.assert_(response.status_code == 302, response.status_code)


class AJAXGetTest(TestCase):
    """Retrieving messages from the server."""

    fixtures = ['test_jqchat.json']

    def setUp(self):
        self.client = Client()
        self.client.login(username='mickey', password='test')

    def test_get_messages(self):
        """Get the latest messages."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0})

        # Basic sanity checks
        self.assert_(response.status_code == 200, response.status_code)
        self.assert_(response['Content-Type'] == 'text/plain; charset=utf-8', response['Content-Type'])
        self.assert_(response['Cache-Control'] == 'no-cache', response['Cache-Control'])

        # Check payload contents
        payload = simplejson.loads(response.content)

        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        # The server returns the Unix time, this will be a number > 0.
        t = payload['time']
        try:
            t = float(t)
        except:
            self.assert_(False, 'Time (%s) should be a number.' % t)

        messages = payload['messages']

        # Check the contents of the first message.
        # Note that messages should always be ordered oldest message first.
        self.assert_(len(messages) == 4)
        self.assert_(messages[0]['text'] == '15:04:29 <strong>mickey</strong>: hello<br />', '#%s#' % messages[0]['text'])

    def test_not_logged_in(self):
        """If the user is not logged in, they cannot access this method."""

        self.client.logout()
        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0})

        self.assert_(response.status_code == 400, response.status_code)

    def test_get_last_message(self):
        """Set the time sent to the server so as to only retrieve the last message for room 1."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 1227629906})

        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)

        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        messages = payload['messages']

        self.assert_(len(messages) == 1)
        self.assert_(messages[0]['text'] == "16:18:27 <strong>mickey</strong>: Yes, let's play!<br />", messages[0]['text'])

    def test_no_messages(self):
        """Set the time sent to the server to after the time of the most recent message."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 1227629910})

        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)

        # Should have a status of 0 (no messages)
        self.assert_(payload['status'] == 0, payload)
        messages = payload['messages']
        self.assert_(len(messages) == 0)

    def test_no_time(self):
        """All requests should include the time."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}))
        self.assert_(response.status_code == 400, response.status_code)

    def test_room_2(self):
        """Retrieve messages for room 2 - should be a different list to room 1.
        Additionally, set the time so as to retrieve only the latest 2 messages in that room -
        the first message gets ignored."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 2}), {'time': 1227629877})

        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)

        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        messages = payload['messages']

        self.assert_(len(messages) == 2)
        self.assert_(messages[0]['text'].startswith('16:17:58'), messages)
        self.assert_(messages[1]['text'].startswith('16:18:02'), messages)


class AJAXPostTest(TestCase):
    """Send new data to the server."""

    fixtures = ['test_jqchat.json']

    def setUp(self):
        self.client = Client()
        self.client.login(username='mickey', password='test')

    def test_post_message(self):
        """Post a new message to the server"""

        response = self.client.post(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0,
                                                         'action': 'postmsg',
                                                         'message': 'rhubarb'})
        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)
        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        messages = payload['messages']

        # Check the contents of the last message - the one that we have just posted.
        # Note that messages should always be ordered oldest message first.
        self.assert_(len(messages) == 5)
        # Remember that we are logged in as Donald Duck.
        self.assert_('mickey' in messages[-1]['text'])
        self.assert_('rhubarb' in messages[-1]['text'])

        # The 'last activity' flag on the room will be updated.
        # (the default value from the test fixture is 0).
        r = Room.objects.get(id=1)
        self.assert_(r.last_activity > 0, r.last_activity)

    def test_not_get(self):
        """If sending new data, we have to use a POST."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0,
                                                         'action': 'postmsg',
                                                         'message': 'rhubarb'})
        self.assert_(response.status_code == 400, response.status_code)

    def test_no_time(self):
        """All requests should include the time."""

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'action': 'postmsg',
                                                         'message': 'rhubarb'})
        self.assert_(response.status_code == 400, response.status_code)

    def test_empty_message(self):
        """Post an empty message to the server - it will be ignored."""

        response = self.client.post(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0,
                                                         'action': 'postmsg',
                                                         'message': '  '})
        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)
        messages = payload['messages']
        # No messages added to the 4 already defined.
        self.assert_(len(messages) == 4)

    def test_XSS(self):
        """Check that chat is protected against cross-site scripting (by disabling html tags)."""

        response = self.client.post(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0,
                                                         'action': 'postmsg',
                                                         'message': '<script>alert("boo!");</script>'})
        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)
        messages = payload['messages']
        self.assert_('&lt;script&gt;' in messages[-1]['text'])
        self.assert_('<script>' not in messages[-1]['text'])

class BehaviourTest(TestCase):
    """Check out how the chat window behaves in different scenarios."""

    fixtures = ['test_jqchat.json']

    def setUp(self):
        self.client = Client()
        self.client.login(username='mickey', password='test')

        self.client2 = Client()
        self.client2.login(username='donald', password='test')

    def test_simultaneous_messages(self):
        """Ensure that messages sent by different users at (virtually) the same time are picked up."""

        response = self.client.post(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0,
                                                         'action': 'postmsg',
                                                         'message': 'rhubarb'})
        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)
        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        mickey_time = payload['time']
        messages = payload['messages']

        self.assert_(len(messages) == 5)
        self.assert_('mickey' in messages[-1]['text'])
        self.assert_('rhubarb' in messages[-1]['text'])

        # Donald also sends a message, at virtually the same time.
        response = self.client2.post(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 0,
                                                         'action': 'postmsg',
                                                         'message': 'crumble'})
        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)
        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        messages = payload['messages']

        # Will pick up the message by Mickey, as well as the one just posted by Donald.
        self.assert_(len(messages) == 6, messages)
        self.assert_('donald' in messages[-1]['text'])
        self.assert_('crumble' in messages[-1]['text'])

        # And the next to last message is the one by Mickey.
        self.assert_('mickey' in messages[-2]['text'])
        self.assert_('rhubarb' in messages[-2]['text'])

        # Mickey immediately requests the latest messages (it could happen...).
        # Note how the time is no longer 0 - it's whatever time was returned from the
        # last AJAX query.
        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': mickey_time})
        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)
        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        messages = payload['messages']

        # Since Mickey last checked, there has been one message posted by Donald.
        self.assert_(len(messages) == 1, messages)
        self.assert_('donald' in messages[-1]['text'])
        self.assert_('crumble' in messages[-1]['text'])

class EventTest(TestCase):
    """Create new events in the room."""

    fixtures = ['test_jqchat.json']

    def setUp(self):
        self.client = Client()
        self.client.login(username='mickey', password='test')

    def test_event(self):
        """Set the time sent to the server so as to only retrieve the last message for room 1.
        Also create a new event, such that we also pick up the event."""

        # Create a new event.
        u = User.objects.get(username='mickey')
        r = Room.objects.get(id=1)
        Message.objects.create_event(u, r, 3)

        response = self.client.get(reverse('jqchat_ajax', kwargs={'id': 1}), {'time': 1227629906})

        self.assert_(response.status_code == 200, response.status_code)

        payload = simplejson.loads(response.content)

        # Should have a status of 1 (there are new messages).
        self.assert_(payload['status'] == 1, payload)

        messages = payload['messages']

        self.assert_(len(messages) == 2)
        self.assert_(messages[0]['text'] == "16:18:27 <strong>mickey</strong>: Yes, let's play!<br />", messages)
        self.assert_("<strong>mickey</strong> <em>has left" in messages[1]['text'], messages)

        # The 'last activity' flag on the room will be updated.
        # (the default value from the test fixture is 0).
        r = Room.objects.get(id=1)
        self.assert_(r.last_activity > 0, r.last_activity)

class DescriptionTest(TestCase):
    """Get and update the description field.
    The description is only in the second test chat window, and is a demo of how to extend
    the chat system.
    """

    fixtures = ['test_jqchat.json']

    def setUp(self):
        self.client = Client()
        self.client.login(username='mickey', password='test')

    def test_get_description(self):
        """For room 1, there is no description."""

        response = self.client.get(reverse('jqchat_test_window_with_description_ajax', kwargs={'id': 1}), {'time': 0})
        self.assert_(response.status_code == 200, response.status_code)
        payload = simplejson.loads(response.content)
        self.assert_(payload.has_key('description') == False)

    def test_get_description2(self):
        """room 2 has a pre-canned description."""

        response = self.client.get(reverse('jqchat_test_window_with_description_ajax', kwargs={'id': 2}), {'time': 0})
        self.assert_(response.status_code == 200, response.status_code)
        payload = simplejson.loads(response.content)
        self.assert_(payload['description'] == 'Enter description here!', payload)


    def test_change_description(self):
        """Change the description for room 2."""

        response = self.client.post(reverse('jqchat_test_window_with_description_ajax', kwargs={'id': 2}), {'time': 0,
                                                                            'action': 'change_description',
                                                                            'description': 'A new description'})
        self.assert_(response.status_code == 200, response.status_code)
        payload = simplejson.loads(response.content)
        self.assert_(payload['description'] == 'A new description', payload)

        # The latest message will be an event.
        messages = payload['messages']
        self.assert_("<strong>mickey</strong>" in messages[-1]['text'], messages)
        self.assert_("description" in messages[-1]['text'], messages)

        # The 'last activity' flag on the room will be updated.
        # (the default value from the test fixture is 0).
        r = Room.objects.get(id=2)
        self.assert_(r.last_activity > 0, r.last_activity)


    def test_XSS(self):
        """Check that chat is protected against cross-site scripting (by disabling html tags)."""

        response = self.client.post(reverse('jqchat_test_window_with_description_ajax', kwargs={'id': 2}), {'time': 0,
                                                                            'action': 'change_description',
                                                                            'description': '<script>alert("boo!");</script>'})
        self.assert_(response.status_code == 200, response.status_code)
        payload = simplejson.loads(response.content)
        self.assert_(payload['description'] == '&lt;script&gt;alert(&quot;boo!&quot;);&lt;/script&gt;', payload)





